#ifndef _P1_IMAGE_SDK_CONVERT_CONFIG_H_
#define _P1_IMAGE_SDK_CONVERT_CONFIG_H_

#include <cstdint>
#include <memory>
#include "C_P1ImageCommonLinkage.h"

#include "P1ImageBitmapImage.hpp"

namespace P1
{
namespace ImageSdk
{
    class ConvertConfigPimpl; // forward decl
    class RawImage; // forward decl

    /**
     * @brief Class that configures the *Convert* pipeline stages.
     * 
     * Currently, you can configure cropping and scaling. The order is fixed.
     */
    class EXPORT_API ConvertConfig
    {
        friend class RawImage;
    private:
        std::unique_ptr<ConvertConfigPimpl> mPimpl;
    public:

        ConvertConfig();
        ConvertConfig(const ConvertConfig& other);

        ConvertConfig& operator=(const ConvertConfig& other);
        ConvertConfig& operator=(ConvertConfig&& other);

        ~ConvertConfig();

        // Accessors

        /** @brief Get any configured crop */
        Rect GetCrop() const;

        bool HasCrop() const;

        bool HasOutputScaling() const;

        /** @brief Get any configured scaling factor, or a negative number, if none if set */
        double GetOutputScale() const;

        /** @brief Get any configured scaling by fixed output width, or a negative number, if none if set */
        int32_t GetOutputWidth() const;

        /** @brief Get any configured scaling by fixed output height, or a negative number, if none if set */
        int32_t GetOutputHeight() const;

        /** @brief The data format of the pixels */
        BitmapFormat GetOutputFormat() const;

        // Setters

        /** @brief Crop image to a given rectangle, in coordinates of the input image */
        ConvertConfig SetCrop(int32_t x, int32_t y, int32_t width, int32_t height);

        /** @brief Crop image to a given rectangle, in coordinates of the input image */
        ConvertConfig SetCrop(const Rect& cropRect);

        /** @brief Scale image relative to the full image size, with aspect ratio enforced */
        ConvertConfig SetOutputScale(double scaleFactor);

        /** @brief Set the scaling to match a given destination width, aspect ratio is still enforced */
        ConvertConfig SetOutputWidth(int32_t destinationWidth);

        /** @brief Set the scaling to match a given destination height, aspect ratio is still enforced */
        ConvertConfig SetOutputHeight(int32_t destinationHeight);

        /** @brief Set the pixel data format of the output image */
        ConvertConfig SetOutputFormat(BitmapFormat format);

        /** @brief Apply your current configuration to a @ref RawImage */
        BitmapImage ApplyTo(const RawImage& image);
    };

}
}

#endif // _P1_IMAGE_SDK_CONVERT_CONFIG_H_